/*  Copyright (c) 2024 by soruce <soruce ~ AT ~ soruce.net>

    This file is part of the Arduino PimientoClockLib Library.

    The Arduino PimientoClockLib Library is free software: you can redistribute it
    and/or modify it under the terms of the GNU General Public License as
    published by the Free Software Foundation, either version 3 of the
    License, or (at your option) any later version.

    The Arduino PimientoClockLib Library is distributed in the hope that it will be
    useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with The Arduino PimientoClockLib Library.  If not, see
    <http://www.gnu.org/licenses/>.
*/

#include "PimientoClockLib.h"

#include "PimientoDisplay.h"

#include <Arduino.h>
#include <uRTCLib.h>
#include <uEEPROMLib.h>
#include <Streaming.h>

static const uint8_t BUZZER_PIN = 6;

uRTCLib rtc(0x68);
uEEPROMLib eeprom(0x57);
PimientoClockLib pimiento;

uint32_t debugRTCCounter = 0;
uint32_t showDisplayDebugInfoCounter = 0;
uint32_t processButtonEventsCounter = 0;

void PimientoClockLib::until(const boolean exitCondition, const uint32_t delay, uint32_t &counter, Callback c) {
  if (!exitCondition) {
    everyMillis(delay, counter, c);
  }
}

void PimientoClockLib::once(uint8_t &state, Callback c) {
  if (state == 0) {
    c();
    state = 1;
  }
}

void PimientoClockLib::repeat(uint16_t times, uint32_t delay, uint32_t &delayCounter, uint16_t &timesCounter, Callback c) {
  if (timesCounter != -1 && timesCounter < times) {
    everyMillis(delay, delayCounter, c);
    timesCounter++;
  }
}

void PimientoClockLib::everyMillis(const uint32_t delay, uint32_t &counter, Callback c) {
  if (millis() - counter >= delay) {
    c();
    counter = millis();
  }
}

void PimientoClockLib::everyFrames(const uint32_t delay, uint32_t &counter, Callback c) {
  if (display.framesUpdatedCounter - counter >= delay) {
    c();
    counter = display.framesUpdatedCounter;
  }
}

void PimientoClockLib::init(EventCallback eventCallback) {

  _eventCallback = eventCallback;

  // initialize serial connection
  Serial.begin(115200);

  // initialize RTC logic
  initRTC();

  // initialize buzzer
  pinMode(BUZZER_PIN, OUTPUT);

  // initialize buttons
  buttons.init(BUTTON_LEFT_PIN, BUTTON_CENTER_PIN, BUTTON_RIGHT_PIN);

  beep(2000, 150);

  // initialize display with desired fps
  display.init(240);
}

void PimientoClockLib::loop() {
 
  waitAndUpdateActiveRow();

  everyMillis(5000, debugRTCCounter, (Callback) &debugRTC);

  // everyMillis(PimientoButtons::SCAN_EVERY_MILLIS, processButtonEventsCounter, (Callback) &processButtonEvents);
  processButtonEvents();

  if (debugEnabled) {
    everyFrames(1000, showDisplayDebugInfoCounter, (Callback) &showDisplayDebugInfo);
  }

  _eventCallback(EventSystemUpdate);
}

void PimientoClockLib::initRTC() {
  URTCLIB_WIRE.begin();
}

void PimientoClockLib::beep(uint16_t frequency, uint32_t duration) {
  tone(BUZZER_PIN, frequency);
  delay(duration);
  noTone(BUZZER_PIN);
}

void PimientoClockLib::waitAndUpdateActiveRow() {

  // TODO for some reason this does not work properly
  // if (display.needsActiveRowUpdate()) {
  //   display.updateActiveRow();
  // }

  while (!display.needsActiveRowUpdate())
    ;
  display.update();
}

// void digitalWriteFast(uint8_t pin, uint8_t value) {
//   if (pin >= 8) {
//     PORTB ^= (-value ^ PORTB) & (1 << (pin % 8));
//   } else {
//     PORTD ^= (-value ^ PORTD) & (1 << (pin % 8));
//   }
// }

void PimientoClockLib::debugRTC() {
  rtc.refresh();
  Serial << "RTC DateTime: " << rtc.year() << '/' << rtc.month() << '/' << rtc.day() << ' ' << rtc.hour() << ':' << rtc.minute() << ':' << rtc.second();
  Serial << " - DOW: " << rtc.dayOfWeek();
  Serial << " - temp: " << (rtc.temp() / 100) << endl;
}

void PimientoClockLib::processButtonEvents() {

    int8_t buttonIndex = buttons.readStates();
    if (buttonIndex == -1) return;

    uint8_t buttonState = buttons.consumeEvent(buttonIndex);

    Serial << "buttonState: " << buttonState << endl;

    if (buttonState == PimientoButtons::EVENT_RELEASED && buttonIndex == BUTTON_LEFT) _eventCallback(EventLeftButtonReleased);
    else if (buttonState == PimientoButtons::EVENT_RELEASED && buttonIndex == BUTTON_RIGHT) _eventCallback(EventRightButtonReleased);
    else if (buttonState == PimientoButtons::EVENT_RELEASED && buttonIndex == BUTTON_CENTER) _eventCallback(EventCenterButtonReleased);
    else Serial << "unsupported button event combination - buttonState: " << buttonState << " - buttonIndex: " << buttonIndex << endl;
}

void PimientoClockLib::showDisplayDebugInfo() {
  display.displayDebugInfo();
}

