/*  Copyright (c) 2024 by soruce <soruce ~ AT ~ soruce.net>

    This file is part of the Arduino PimientoClockLib Library.

    The Arduino PimientoClockLib Library is free software: you can redistribute it
    and/or modify it under the terms of the GNU General Public License as
    published by the Free Software Foundation, either version 3 of the
    License, or (at your option) any later version.

    The Arduino PimientoClockLib Library is distributed in the hope that it will be
    useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with The Arduino PimientoClockLib Library.  If not, see
    <http://www.gnu.org/licenses/>.
*/

#ifndef PIMIENTO_DISPLAY_H
#define PIMIENTO_DISPLAY_H

#include <Arduino.h>
#include "SparkFun_Tlc5940.h"
#include "IFont.h"

const uint8_t BLINK = 128;

const uint8_t DISPLAY_WIDTH = 32;
const uint8_t DISPLAY_HEIGHT = 8;

const uint8_t FRAMEBUFFER_WIDTH = DISPLAY_WIDTH;
const uint8_t FRAMEBUFFER_HEIGHT = DISPLAY_HEIGHT;

const uint8_t MAX_BRIGHTNESS = 8;

const int DRIVER_BLANK_PIN = 10; 	// Arduino D10
const int SHIFT_LATCH_PIN = 6;    // Arduino D6
const int SHIFT_DATA_PIN = 7;     // Arduino D7
const int SHIFT_CLOCK = 8;        // Arduino D8

const bool debugEnabled = true;

class PimientoDisplay {

  public:

    void init(uint16_t desiredFps);
    void setEnabled(bool b);

    uint8_t cursorX = 0;
    uint8_t cursorY = 0;

    // low-level stuff
    uint8_t frameBuffer[FRAMEBUFFER_WIDTH * FRAMEBUFFER_HEIGHT];
    unsigned long rowsUpdatedCounter = 0;
    unsigned long framesUpdatedCounter = 0;

    void update();

    bool needsActiveRowUpdate();
    void debugFrameBuffer();
    
    void displayDebugInfo();
    
    // graphics library
    void clearAll();
    void clearPage();

    void setFrameBufferPixel(int frameBufferX, int frameBufferY, uint8_t color);

    void setPixel(int x, int y, uint8_t color);

    void drawRectangle(int x, int y, int width, int height, uint8_t color);
    void fillRectangle(int x, int y, int width, int height, uint8_t color);
    void drawHorizontalLine(int x, int y, int width, uint8_t color);
    void drawVerticalLine(int x, int y, int height, uint8_t color);

    // paging related
    uint8_t getPage();
    void setPage(uint8_t page);
    
    // blinking logic
    void setBlinkSpeed(uint16_t blinkOnDelayMillis, uint16_t blinkOffDelayMillis);

    // framebuffer
    void moveFrameBufferContentsDown();
    void moveFrameBufferContentsRight();

    void copyRowContents(int srcRow, int destRow);
    void copyColumnContents(int srcColumn, int destColumn);

    // bitmap font
    uint8_t drawChar(int x, int y, char c, uint8_t color);
    void drawString(int x, int y, char str[], uint8_t color);
    void drawInt(int x, int y, int number, uint8_t color);
    
    // bitmap font with cursor
    void resetCursor();
    void setCursor(int x, int y);
    void moveCursor(int x, int y);
    
    uint8_t printChar(char c, uint8_t color);
    void printString(char str[], uint8_t color);
    void printInt(int number, uint8_t color);
    void printInt(int number, int numberOfDigits, uint8_t color);

    uint8_t getFont();
    void setFont(uint8_t fontIndex);

    // brightness between 0 and 8 (inclusive)
    void setBrightness(uint8_t brightness);
    uint8_t getBrightness();

  private:
    void setRefreshRate(uint16_t desiredFps);
    void updateTlcData();
    void updateActiveRow();

    void pulseHigh(int pin);
    bool isBitSet(uint8_t value, int bitIndex);

    // bitmap font
    uint8_t getCharWidth(char c);

    // blinking logic
    void updateBlink();
    uint16_t blinkOnDelayMillis = 0;
    uint16_t blinkOffDelayMillis = 0;
    unsigned long blinkCounter = 0;
    bool blinkOn = true;

    // row update logic
    uint8_t activeRow = 0;    // row is from 0..7
    unsigned long lastRowUpdateMillis = 0;
    unsigned long refreshDelayMillis = 0;

    uint8_t _brightness = MAX_BRIGHTNESS;
    uint8_t currentPage = 0;
};

extern PimientoDisplay display;

#endif /* PIMIENTO_DISPLAY_H */
