/*  Copyright (c) 2024 by soruce <soruce ~ AT ~ soruce.net>

    This file is part of the Arduino PimientoClockLib Library.

    The Arduino PimientoClockLib Library is free software: you can redistribute it
    and/or modify it under the terms of the GNU General Public License as
    published by the Free Software Foundation, either version 3 of the
    License, or (at your option) any later version.

    The Arduino PimientoClockLib Library is distributed in the hope that it will be
    useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with The Arduino PimientoClockLib Library.  If not, see
    <http://www.gnu.org/licenses/>.
*/

/**
 * A basic clock that displays time in the "HH:MM:SS" format.
 * In the main mode, pressing the Left or Right button will decrease or increase the display brightness.
 * In the main mode, pressing the Center button will cycle through the time setting modes (set hours / set minutes / reset seconds).
 * In the time setting modes, pressing the Left or Right button will change the selected (blinking) time setting.
*/

#include <PimientoClockLib.h>
#include <Streaming.h>

const uint8_t StateDefault = 0;
const uint8_t StateSimpleClock = 1;
const uint8_t StateSetHours = 2;
const uint8_t StateSetMinutes = 3;
const uint8_t StateResetSeconds = 4;

uint8_t currentState = StateDefault;
bool stateInitNeeded = true;
uint32_t stateUpdateCounter = 0;

extern void updateState(uint8_t event = EventSystemUpdate);

// ----

void setup() {
  pimiento.init(updateState);
}

void loop() {
  pimiento.loop();
}

// ----

void setState(uint8_t newState) {
  stateInitNeeded = true;
  stateUpdateCounter = 0;
  currentState = newState;
}

void updateState(uint8_t event) {

  if (event != EventSystemUpdate) {
    Serial << "updateState - currentState: " << currentState << " - event: " << event << endl;
  }

  switch (currentState) {
    case StateDefault:
      if (event == EventSystemUpdate) PimientoClockLib::everyMillis(250, stateUpdateCounter, loopDrawClock);
      else if (event == EventCenterButtonReleased) setState(StateSetHours);
      else if (event == EventLeftButtonReleased) incBrightness(-1);
      else if (event == EventRightButtonReleased) incBrightness(1);
      break;
    case StateSetHours:
      if (event == EventSystemUpdate) PimientoClockLib::everyMillis(250, stateUpdateCounter, loopDrawClock);
      else if (event == EventLeftButtonReleased) incHour(-1);
      else if (event == EventRightButtonReleased) incHour(1);
      else if (event == EventCenterButtonReleased) setState(StateSetMinutes);
      break;
    case StateSetMinutes:
      if (event == EventSystemUpdate) PimientoClockLib::everyMillis(250, stateUpdateCounter, loopDrawClock);
      else if (event == EventLeftButtonReleased) incMinute(-1);
      else if (event == EventRightButtonReleased) incMinute(1);
      else if (event == EventCenterButtonReleased) setState(StateResetSeconds);
      break;
    case StateResetSeconds:
      if (event == EventSystemUpdate) PimientoClockLib::everyMillis(250, stateUpdateCounter, loopDrawClock);
      else if (event == EventLeftButtonReleased) resetSeconds();
      else if (event == EventRightButtonReleased) resetSeconds();
      else if (event == EventCenterButtonReleased) setState(StateDefault);
      break;
    default:
      Serial << "unsupported - state: " << currentState << ", event: " << event << endl;
  }
}

void incBrightness(int8_t increment) {

  int8_t brightness = (display.getBrightness() + increment);
  if (brightness == -1) {
    brightness = MAX_BRIGHTNESS;
  }
  if (brightness > MAX_BRIGHTNESS) {
    brightness = 0;
  }
  display.setBrightness(brightness);
}

void incHour(int8_t increment) {
  rtc.refresh();
  int8_t newHour = (rtc.hour() + increment) % 24;
  rtc.set(rtc.second(), rtc.minute(), newHour, rtc.dayOfWeek(), rtc.day(), rtc.month(), rtc.year());
}

void incMinute(int8_t increment) {
  rtc.refresh();
  int8_t newMinute = (rtc.minute() + increment) % 60;
  rtc.set(rtc.second(), newMinute, rtc.hour(), rtc.dayOfWeek(), rtc.day(), rtc.month(), rtc.year());
}

void resetSeconds() {
  rtc.refresh();
  rtc.set(0, rtc.minute(), rtc.hour(), rtc.dayOfWeek(), rtc.day(), rtc.month(), rtc.year());
}

void loopDrawClock() {

  rtc.refresh();

  display.clearAll();
  display.setCursor(1, 1);

  uint8_t hoursColor = (currentState == StateSetHours) ? MAIN_COLOR + BLINK : MAIN_COLOR;
  display.printInt(rtc.hour(), 2, hoursColor);

  display.moveCursor(1, 0);

  uint8_t minutesColor = (currentState == StateSetMinutes) ? MAIN_COLOR + BLINK : MAIN_COLOR;
  display.printInt(rtc.minute(), 2, minutesColor);

  display.moveCursor(1, 0);

  uint8_t secondsColor = (currentState == StateResetSeconds) ? MAIN_COLOR + BLINK : MAIN_COLOR;
  display.printInt(rtc.second(), 2, secondsColor);
}
